import csv
import sys
import os
import json
import pkg_resources
from psycopg2 import connect, extensions
from datetime import datetime, date, timedelta
from croniter import croniter
from pathlib import Path

import p2.tools

db = os.getenv('DB_P2STORE_DBNAME')

if not db:
    raise Exception("DB_P2STORE_DBNAME not set")


def exec_db(database, cmd):
    conn = connect(
            dbname=database,
            host='localhost',  # we force it to work on localhost...
            user=os.getenv('DB_P2STORE_USER', 'postgres'),
            password=os.getenv('DB_P2STORE_PASSWORD')
            )
    autocommit = extensions.ISOLATION_LEVEL_AUTOCOMMIT
    conn.set_isolation_level(autocommit)
    cursor = conn.cursor()
    cursor.execute(cmd)
    cursor.close()
    conn.close()


def import_p2store_schema(database: str):
    conn = connect(
            dbname=database,
            host='localhost',  # we force it to work on localhost...
            user=os.getenv('DB_P2STORE_USER', 'postgres'),
            password=os.getenv('DB_P2STORE_PASSWORD')
            )
    autocommit = extensions.ISOLATION_LEVEL_AUTOCOMMIT
    conn.set_isolation_level(autocommit)
    cursor = conn.cursor()
    path = p2.tools.__path__[0]
    root = '/'.join((path.split('/'))[:-2])
    with open(f"{root}/schema/p2store-schema.sql", "r") as schema:
        cursor.execute(schema.read())
        schema.close()


def populate_resources():
    exec_db(db, 'alter table resources disable trigger all;')
    exec_db(db, 'DELETE FROM resources CASCADE;')
    exec_db(db, """SELECT setval(pg_get_serial_sequence('resources', 'id'),
                coalesce(max(id),1), false)
            FROM resources;""")
    insert_records_from_csv_file('resources',
                                 get_csv_records('data/resources.csv'))
    exec_db(db, 'alter table resources enable trigger all;')


def populate_slack_channels():
    exec_db(db, 'DELETE FROM slack_channels;')
    exec_db(db, """
        INSERT INTO slack_channels (channel, url)
        VALUES ('chaos_engineering_notifications',
        'https://hooks.slack.com/services/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');""")

def populate_reports():
    exec_db(db, 'alter table reports disable trigger all;')
    exec_db(db, 'DELETE FROM reports CASCADE;')
    exec_db(db,
            """
            SELECT setval(pg_get_serial_sequence('reports', 'id'), coalesce(max(id),1), false)
            FROM reports;
            """)
    insert_records_from_csv_file('reports', get_csv_records('data/reports.csv'))
    insert_records_from_csv_file('report_entries',
                                 get_csv_records('data/report_entries.csv'))
    exec_db(db, 'alter table reports enable trigger all;')


def populate_campaigns():
    exec_db(db, 'alter table test_campaigns disable trigger all;')
    exec_db(db, 'DELETE FROM test_campaigns CASCADE;')
    exec_db(db, 'alter table test_campaigns enable trigger all;')
    insert_records_from_csv_file('test_campaigns',
                                 get_csv_records('data/test_campaigns.csv'))


def populate_config():
    exec_db(db, 'alter table config disable trigger all;')
    exec_db(db, 'DELETE FROM config;')
    insert_records_from_csv_file('config', get_csv_records('data/config.csv'))


def _populate_templates():
    text_template = """
{#
-------------------------------
This is just a test template...
-------------------------------
#}
{
  "version": "1.0.0",
  "title": "Create and delete dir {{ target.dir }}",
  "description": "Create and delete dir {{ target.dir }}",
  "tags": [ "mkdir", "rmdir", "directory", "{{ target.dir }}" ],

  {% if contributions -%}
    "contributions": {{ contributions|tojson(indent=2) }},
  {%- endif -%}

  {% if configuration -%}
    "configuration": {{ configuration|tojson(indent=2) }},
  {%- endif -%}

  {% if secrets -%}
    "secrets": {{ secrets|tojson(indent=2) }},
  {%- endif -%}

  {% if controls -%}
    "controls": {{ controls|tojson(indent=2) }},
  {%- endif -%}

    "steady-state-hypothesis": {
        "title": "The file must be around first",
        "probes": [
            {
                "type": "probe",
                "name": "file-must-exist",
                "tolerance": true,
                "provider": {
                    "type": "python",
                    "module": "os.path",
                    "func": "exists",
                    "arguments": {
                      "path": "{{ target.base_dir }}"
                    }
                }
            }
        ]
    },
    "method": [
        {
            "type": "action",
            "name": "create-file-be-gone",
            "provider": {
                "type": "python",
                "module": "os",
                "func": "mkdir",
                "arguments": {
                  "path": "{{ target.dir }}"
                }
            },
            "pauses": {
                "after": 5
            }
        },
        {
            "type": "action",
            "name": "remove-file-be-gone",
            "provider": {
                "type": "python",
                "module": "os",
                "func": "rmdir",
                "arguments": {
                  "path": "{{ target.dir }}"
                }
            },
            "pauses": {
                "after": 5
            }
        },
        {
            "ref": "file-must-exist"
        }
    ]
}"""
    json_template = json.dumps(text_template)
    stmt = """
INSERT INTO experiments (filename, tag, data, description, type, profile)
VALUES (
    'create-remove-dir-template',
    'create-remove-dir',
    '{}',
    'Removing a dir from under our feet is forgivable',
    'template',
    '{{ \"tags\": [\"path\"],
        \"title\": \"Create and remove a dir\",
        \"contributions\": \"foo bar\",
        \"description\": \"Removing a dir from under our feet is forgivable\",
        \"weight\": 1.0
    }}'
);""".format(json_template)
    exec_db(db, stmt)


def populate_experiments():
    exec_db(db, 'alter table experiments disable trigger all;')
    exec_db(db, 'alter table exp_logs disable trigger all;')
    exec_db(db, 'DELETE FROM exp_logs;')
    exec_db(db, 'DELETE FROM experiments;')
    _populate_templates()

    data = get_csv_records('data/experiments.csv')
    insert_records_from_csv_file('experiments', data)

    exec_db(db, 'alter table experiments enable trigger all;')
    exec_db(db, 'alter table exp_logs enable trigger all;')


def populate_exp_session():
    """ sessions history:
    1: exp 1 1 month ago status completed
    2: exp 1 2 weeks ago status completed
    3: exp 1 1 week ago status failed
    4: exp 2 3 weeks ago status completed
    5: exp 3 2 months ago status completed
    6: exp 4 1 months ago status completed
    7: exp 5 1 months ago status completed
    8: exp 5 1 week ago status completed
    9: exp 1 the first future day possible scheduled
    10:exp 2 the first future day possible scheduled
    11:create-remove-dir-1234: session on targer dir running now...
    12:create-remove-dir-template-1234: session on targer dir running now...
    """
    today = date.today()
    one_month_ago = today - timedelta(weeks=4.3)
    two_months_ago = today - timedelta(weeks=8.6)
    one_week_ago = today - timedelta(weeks=1)
    two_weeks_ago = today - timedelta(weeks=2)
    three_weeks_ago = today - timedelta(weeks=3)

    future_day = today+timedelta(days=2)

    exec_db(db, 'alter table exp_session disable trigger all;')
    exec_db(db, 'DELETE FROM exp_session CASCADE;')
    session = {
            'exp_id': '1',
            'campaign_id': 1,
            'resource_id': 8,
            'filepath': 'exp_1',
            'schedule':"0 0"+' '+str(one_month_ago.day)+' '+str(one_month_ago.month)+' *',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % one_month_ago.day,
            'month': "%02d" % one_month_ago.month,
            'dow': '*',
            'start_time': str(one_month_ago)+ " 00:00:00.0",
            'end_time': str(one_month_ago)+ " 00:20:00.0",
            'status': 'completed',
            'context': 'minikube',
            'year': one_month_ago.year,
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns1"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, resource_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{resource_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))

    session = {
            'exp_id': '2',
            'campaign_id': 1,
            'resource_id': 8,
            'context': 'minikube',
            'filepath': 'exp_1',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % two_weeks_ago.day,
            'month': "%02d" % two_weeks_ago.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(two_weeks_ago.day)+' '+str(two_weeks_ago.month)+' *',
            'year': two_weeks_ago.year,
            'start_time': str(two_weeks_ago)+ " 00:00:00.0",
            'end_time': str(two_weeks_ago)+ " 00:20:00.0",
            'status': 'completed',
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns1"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, resource_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{resource_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))

    session = {
            'exp_id': '3',
            'campaign_id': 1,
            'resource_id': 8,
            'context': 'minikube',
            'filepath': 'exp_1',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % one_week_ago.day,
            'month': "%02d" % one_week_ago.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(one_week_ago.day)+' '+str(one_week_ago.month)+' *',
            'year': one_week_ago.year,
            'start_time': str(one_week_ago)+ " 00:00:00.0",
            'end_time': str(one_week_ago)+ " 00:20:00.0",
            'status': 'failed',
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns1"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, resource_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{resource_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))


    session = {
            'exp_id': '4',
            'campaign_id': 1,
            'context': 'minikube',
            'filepath': 'exp_2',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" %  three_weeks_ago.day,
            'month': "%02d" % three_weeks_ago.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(three_weeks_ago.day)+' '+str(three_weeks_ago.month)+' *',
            'year': three_weeks_ago.year,
            'start_time': str(three_weeks_ago)+ " 00:00:00.0",
            'end_time': str(three_weeks_ago)+ " 00:20:00.0",
            'status': 'failed',
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns2-int-nft"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))


    session = {
            'exp_id': '5',
            'campaign_id': 1,
            'context': 'minikube',
            'filepath': 'exp_3',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % two_months_ago.day,
            'month': "%02d" % two_months_ago.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(two_months_ago.day)+' '+str(two_months_ago.month)+' *',
            'year': two_months_ago.year,
            'start_time': str(two_months_ago)+ " 00:00:00.0",
            'end_time': str(two_months_ago)+ " 00:20:00.0",
            'status': 'completed',
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns3-int-nft"
                    }
                })
        }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))


    session = {
            'exp_id': '6',
            'campaign_id': 1,
            'context': 'minikube',
            'filepath': 'exp_4',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % one_month_ago.day,
            'month': "%02d" % one_month_ago.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(one_month_ago.day)+' '+str(one_month_ago.month)+' *',
            'year': one_month_ago.year,
            'start_time': str(one_month_ago)+ " 00:00:00.0",
            'end_time': str(one_month_ago)+ " 00:20:00.0",
            'status': 'completed',
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns3-int-nft"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))


    session = {
            'exp_id': '7',
            'campaign_id': 1,
            'context': 'minikube',
            'filepath': 'exp_5',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % one_month_ago.day,
            'month': "%02d" % one_month_ago.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(one_month_ago.day)+' '+str(one_month_ago.month)+' *',
            'year': one_month_ago.year,
            'start_time': str(one_month_ago)+ " 00:00:00.0",
            'end_time': str(one_month_ago)+ " 00:20:00.0",
            'status': 'completed',
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns4"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))


    session = {
            'exp_id': '8',
            'campaign_id': 1,
            'context': 'context-2',
            'filepath': 'exp_5',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % one_week_ago.day,
            'month': "%02d" % one_week_ago.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(one_week_ago.day)+' '+str(one_week_ago.month)+' *',
            'year': one_week_ago.year,
            'start_time': str(one_week_ago)+ " 00:00:00.0",
            'end_time': str(one_week_ago)+ " 00:20:00.0",
            'status': 'completed',
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns4"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))


    session = {
            'exp_id': '9',
            'campaign_id': 1,
            'resource_id': 8,
            'context': 'minikube',
            'filepath': 'exp_1',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % future_day.day,
            'month': "%02d" % future_day.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(future_day.day)+' '+str(future_day.month)+' *',
            'year': future_day.year,
            'start_time': str(future_day)+ " 00:00:00.0",
            'end_time': str(future_day)+ " 00:20:00.0",
            'status': 'scheduled',
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns1"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, resource_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{resource_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))


    session = {
            'exp_id': '10',
            'campaign_id': 1,
            'context': 'minikube',
            'filepath': 'exp_2',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % future_day.day,
            'month': "%02d" % future_day.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(future_day.day)+' '+str(future_day.month)+' *',
            'year': future_day.year,
            'start_time': str(future_day)+ " 00:00:00.0",
            'end_time': str(future_day)+ " 00:20:00.0",
            'status': 'scheduled',
            'profile':json.dumps({
                'target': {
                        "kind":"namespace",
                        "name":"ns2-int-nft"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, campaign_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{campaign_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))

    session = {
            'exp_id': 'create-remove-dir-1234',
            'context': 'os',
            'filepath': 'create-remove-dir',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % today.day,
            'month': "%02d" % today.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(today.day)+' '+str(today.month)+' *',
            'year': today.year,
            'start_time': str(today)+ " 00:00:00.0",
            'end_time': str(today)+ " 00:20:00.0",
            'status': 'scheduled',
            'profile':json.dumps({
                'target': {
                        "kind":"dir",
                        "base_dir":"/tmp",
                        "dir":"/tmp/foo"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))

    session = {
            'exp_id': 'create-remove-dir-template-1234',
            'context': 'os',
            'filepath': 'create-remove-dir-template',
            'mins': '0',
            'hour': '0',
            'dom': "%02d" % today.day,
            'month': "%02d" % today.month,
            'dow': '*',
            'schedule': "0 0"+' '+str(today.day)+' '+str(today.month)+' *',
            'year': today.year,
            'start_time': str(today)+ " 00:00:00.0",
            'end_time': str(today)+ " 00:20:00.0",
            'status': 'scheduled',
            'profile':json.dumps({
                'target': {
                        "kind":"dir",
                        "dir":"/tmp/foo",
                        "base_dir": "/tmp"
                    }
                })
            }
    iter = croniter(session['schedule'], datetime.now())
    session['next_run_time'] = str(iter.get_next(datetime))
    exec_db(db,
            """
            INSERT INTO exp_session (
                exp_id, filepath, schedule, mins, hour, dom, month, dow,
                start_time, end_time, status, context, year, next_run_time, profile
            ) VALUES ('{exp_id}', '{filepath}', '{schedule}',
                '{mins}', '{hour}', '{dom}', '{month}', '{dow}',
                '{start_time}', '{end_time}', '{status}', '{context}',
                '{year}', '{next_run_time}', '{profile}');""".format(**session))


def populate_logs():
    exec_db(db, 'alter table exp_logs disable trigger all;')
    exec_db(db, 'DELETE FROM exp_logs;')
    insert_records_from_csv_file('exp_logs',
                                 get_csv_records('data/exp_logs.csv'))
    exec_db(db, 'alter table exp_logs enable trigger all;')

def createDB(db: str):
    """Creates a DB with the passed name and with the p2store schema """
    try:
        exec_db('postgres', f'DROP DATABASE IF EXISTS "{db}"')
        exec_db('postgres', f'CREATE DATABASE "{db}" WITH TEMPLATE p2store;')
        return True
    except Exception as error:
        print(error)
        return False

def createDB_from_schema(db: str):
    try:
        exec_db('postgres', f'DROP DATABASE IF EXISTS "{db}"')
        exec_db('postgres', f"CREATE DATABASE {db} ENCODING = 'UTF8'")
        import_p2store_schema(db)
    except Exception as error:
        print(error)


def destroyDB(db:str):
    try:
        exec_db('postgres', f'DROP DATABASE IF EXISTS "{db}"')
    except Exception as error:
        print(error)

def commit_update(updates_file: str, file: str):
    try:
        with open(updates_file, 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=';')
            return writer.writerow([file, datetime.now().isoformat()])
    except Exception as e:
        raise Exception(f"Error committing file {file} onto {updates_file}: {e}")

def get_csv_records(filename: str):
    """Parses a CSV filename, returns a dict with table columns and records"""
    columns = []
    records = []
    data_file = pkg_resources.resource_filename(__name__, filename)
    with open(data_file) as csvfile:
        reader = csv.reader(
            csvfile,
            delimiter='|',
            quotechar=None,
            # quoting=csv.QUOTE_NONE,
            doublequote=True
        )
        columns = next(reader)
        for record in reader:
            records.append(record)
    return {'columns': columns, 'records':records}

def insert_records_from_csv_file(table:str, data:dict):
    """Insert all records in data into the passed table"""
    for record in data['records']:
        stmt = "INSERT INTO {} ({}) VALUES ({});".format(
            table,
            ",".join(data['columns']),
            ",".join(record))
        exec_db(db, stmt)


def get_data_from_csv_files():
    """Reads all records from all files stored under the data
    directory. The file name is the table name. Each file first raw
    identifies the columns to use.

    Returns a dict with table names as keys and a dict with columns and records as
    values.
    """
    tables = dict()
    try:
        files = pkg_resources.resource_listdir(__name__, 'data')
        for file in files:
            data_file = pkg_resources.resource_filename(__name__, f"data/{file}")
            with open(data_file) as f:
                table = Path(data_file).stem
                tables[table] = get_csv_records(f"data/{file}")
        return tables
    except Exception as e:
        print(f"Error loading records from CSV files: {e}")
        return None

def check_updates(updates_dir: str=None, updates_file: str=None):
    """Gets the list of files in updates_dir, parses csv file updates_file
    Return first file in updates_dir not present in updates_file
    """
    updates_dir = updates_dir or 'schema/updates'
    updates_file = updates_file or 'schema/p2store-updates.csv'
    update_files = os.scandir(updates_dir)
    update_file_names = sorted([n.name for n in update_files])
    with open(updates_file) as f:
        reader = csv.reader(f, delimiter=';')
        files_updated = list()
        for file in reader:
            files_updated.append(file[0])

        for file in update_file_names:
            if file not in files_updated:
                return file

def setUpDB():
    try:
        populate_config()
        populate_resources()
        populate_reports()
        populate_campaigns()
        populate_experiments()
        populate_exp_session()
        populate_logs()
        populate_slack_channels()

        print("{} db populated on {}".format(db, os.getenv('DB_P2STORE_HOST')))
    except Exception as error:
        print(error)


def tearDownDB():
    try:
        exec_db(db, 'DELETE FROM resources;');
        exec_db(db, 'DELETE FROM exp_logs;');
        exec_db(db, 'DELETE FROM config CASCADE;');
        exec_db(db, 'DELETE FROM reports CASCADE;');
        exec_db(db, 'DELETE FROM experiments CASCADE;');
        exec_db(db, 'DELETE FROM k8s_resources CASCADE;');
        exec_db(db, 'DELETE FROM exp_session CASCADE;');
        exec_db(db, 'DELETE FROM test_campaigns CASCADE;');
        print("{} db cleared on {}".format(db, os.getenv('DB_P2STORE_HOST')))
    except:
        print("Unexpected error:", sys.exc_info()[0])


def set_db_name( database: str ):
    global db
    db = database
    os.environ['DB_P2STORE_DBNAME'] = database

if __name__ == '__main__':
    setUpDB()
