import os
import re
import unittest
from psycopg2 import connect, sql, Error
import psycopg2.extras
import logging

from p2.tools.prepare_db import set_db_name, createDB, createDB_from_schema, setUpDB, tearDownDB, commit_update

def encode_all(data):
    if isinstance(data, dict):
        return encode_fields(data)
    if isinstance(data, (list, tuple)):
        encoded = list()
        for item in data:
            encoded.append(json.dumps(item) if isinstance(item, (dict, list)) else item)
        return tuple(encoded)

class PostgresClient:

    def __init__(self, user=None, password=None, db=None, host=None, port=5432):
        self.log = logging.getLogger(__name__)
        try:
            self.connection = connect(
                connection_factory=psycopg2.extras.LoggingConnection,
                user     = user or os.environ.get('DB_P2STORE_USER', 'postgres'),
                password = password or os.environ.get('DB_P2STORE_PASSWORD'),
                host     = host or os.environ.get('DB_P2STORE_HOST'),
                port     = port or os.environ.get('DB_P2STORE_PORT'),
                database = db or os.environ.get('DB_P2STORE_DBNAME', 'p2store'),
                connect_timeout = 10
                )
            self.connection.initialize(self.log)
        except psycopg2.Error as e:
            self.log.error("Error getting a connection: {}".format(e))
            raise e

    def execute(self, stmt, *data):
        data = encode_all(data)
        self.log.debug(stmt)
        try:
            cursor = self.connection.cursor(cursor_factory = psycopg2.extras.DictCursor)
            cursor.execute(stmt, data)

            if re.search('select', str(stmt), re.IGNORECASE):
                record = cursor.fetchall()
                return(record)
            else:
                self.connection.commit()
                return(cursor.rowcount)
        except (Exception, Error) as error :
            self.log.error(f"Stmt: {stmt}, Data: {data}, Error: {error}")
            raise error


    def select( self, table,
            limit: int=None,
            offset: int=None,
            fetch: list=None,
            sort: str=None,
            **filter ):
        """ Returns all records from table that match the filter """

        return self.execute(*(self._build_query(
            table=table,
            limit=limit,
            offset=offset,
            fetch=fetch,
            sort=sort,
            filter=filter)))


    def _build_query(self, table: str, limit: int=None, offset: int=None, fetch: list=None,
                     sort: str=None, filter: dict=None ) -> str:

        fetchers = ", ".join(fetch) if fetch else '*'
        values = list()
        if filter:
            filter_str = ""
            for key in sorted(filter.keys()):
                (filter_str, values) = self._build_where_clause(filter_str, values, key, filter[key])

            query = "SELECT {} FROM {} WHERE {}".format(fetchers, table, filter_str)
        else:
            query = "SELECT {} FROM {}".format(fetchers, table)
        if sort:
            query += " ORDER BY {}".format(sort)
        if limit is not None:
            query += " LIMIT %s "
            values.append(limit)
        if offset is not None:
            query += " OFFSET %s"
            values.append(offset)
        return (query, *values)


class Test_preparedb(unittest.TestCase):

    def SetUp(self):
        setUpDB()

    def TearDown(self):
        tearDownDB()

    def test_001_create_test_db(self):
        db = os.getenv('DB_P2STORE_DBNAME')
        createDB_from_schema(db)
        client = PostgresClient()
        records = client.select(table='experiments')
        self.assertEqual(len(records), 0)

    def test_002_setup_test_db(self):
        setUpDB()
        client = PostgresClient()
        records = client.select(table='experiments')
        self.assertEqual(len(records), 7)

    def test_003_commit_update(self):
        ret = commit_update(updates_file='scripts/p2store_updates.csv', file='foo.sql')
        self.assertGreater(ret, 0)

    def test_004_setup_teardown(self):
        setUpDB()
        client = PostgresClient()
        records = client.select(table='experiments')
        self.assertEqual(len(records), 7)
        tearDownDB()
        records = client.select(table='experiments')
        self.assertEqual(len(records), 0)
